# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

from xml.dom import minidom

import Globals
from Products.Formulator import Form, TALESField, MethodField
from types import ListType, TupleType, DictType, IntType, StringType, UnicodeType
MethodType = type(MethodField.Method(''))
known_types = (ListType, TupleType, DictType, IntType, StringType, UnicodeType, MethodType)

file_path = Globals.package_home(globals())
LATIN1 = "latin1"
UTF8 = "UTF-8"

def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return str(rc.strip().encode(LATIN1))

class NullField:
    pass

class FormulatorImporter:
    def importXML(self, file_or_filename):
        if StringType is type(file_or_filename):
            f = open(file_path + '/' + file_or_filename + '.xml')
        else:
            f = file_or_filename
        doc = minidom.parse(f)
        f.close()
        form = self.parseFormulatorForm(doc)
        return form
    def parseFormulatorForm(self, doc):
        form_node = doc.getElementsByTagName("form")[0]
        form = self.handleForm(form_node)
        return form
        
    def handleForm(self, form_node):
        settings_node = form_node.getElementsByTagName("settings")[0]
        settings = self.handleSettings(settings_node)
        form_id = str(form_node.getAttribute('id'))
        form_title = str(form_node.getAttribute('title'))
        form = Form.ZMIForm(form_id, form_title)
        while settings:
            key, value = settings.popitem()
            setattr(form, key, value)

        groups_node = form_node.getElementsByTagName("groups")[0]
        groups, group_list, fields = self.handleGroups(groups_node)

        for field in fields:
            form.manage_addField(str(field.id), field.id, field.meta_type)
            fld = form[field.id]
            if hasattr(field, 'values'):
                fld.values = field.values
            if hasattr(field, 'tales'):
                fld.tales = field.tales
            if hasattr(field, 'overrides'):
                fld.overrides = field.overrides
            if hasattr(field, 'messages'):
                fld.message_values = field.messages
        form.groups = groups
        form.group_list = group_list
        return form
        
        
    def handleSettings(self, settings_node):
        setting_nodes = settings_node.getElementsByTagName("setting")
        settings = {}
        for setting_node in setting_nodes:
            name = str(setting_node.getAttribute("name"))
            value = self.handleValue(setting_node)
            settings[name] = value
        return settings
        

    def handleGroups(self, groups_node):
        group_nodes = groups_node.getElementsByTagName("group")
        groups = {}
        group_list = []
        all_fields = []
        for group_node in group_nodes:
            name = str(group_node.getAttribute("name"))
            group_list.append(name)
            fields_node = group_node.getElementsByTagName("fields")[0]
            field_keys, field_values = self.handleFields(fields_node)
            all_fields.extend(field_values)
            groups[name] = field_keys
        return groups, group_list, all_fields
            
            
            
    def handleFields(self, fields_node):
        field_nodes = fields_node.getElementsByTagName("field")
        field_values = []
        field_keys = []
        for field_node in field_nodes:
            field = self.handleField(field_node)
            field_values.append(field)
            field_keys.append(field.id)
        return field_keys, field_values

    def handleField(self, field_node):
        field = NullField()
        field_id = str(field_node.getAttribute("name"))
        meta_type = str(field_node.getAttribute("meta_type"))
        field.id = field_id
        field.meta_type = meta_type
        values_nodes = field_node.getElementsByTagName("values")
        if values_nodes:
            values_node = values_nodes[0]
            self.handleValues(values_node, field)
        tales_nodes = field_node.getElementsByTagName("tales")
        if tales_nodes:
            tales_node = tales_nodes[0]
            self.handleTales(tales_node, field)
        overrides_nodes = field_node.getElementsByTagName("overrides")
        if overrides_nodes:
            overrides_node = overrides_nodes[0]
            self.handleOverrides(overrides_node, field)
        messages_nodes = field_node.getElementsByTagName("messages")
        if messages_nodes:
            messages_node = messages_nodes[0]
            self.handleMessages(messages_node, field)
        return field
        
    def handleValues(self, values_node, field):
        value_nodes = values_node.getElementsByTagName("value")
        field.values = {}
        for value_node in value_nodes:
            name = str(value_node.getAttribute("name"))
            value = self.handleValue(value_node)
            field.values[name] = value

    def handleValue(self, value_node):
        type_name = str(value_node.getAttribute("type"))
        t = StringType
        for some_type in known_types:
            if some_type.__name__ == type_name:
                t = some_type
                break
        
        if ListType is t or TupleType is t:
            value = []
            list_node = value_node.getElementsByTagName(t.__name__)[0]
            item_nodes = list_node.getElementsByTagName("%s_item" % t.__name__)
            for item_node in item_nodes:
                v = self.handleValue(item_node)
                value.append(v)
            if TupleType is t:
                value = tuple(value)
        elif DictType is t:
            value = {}
            item_nodes = value_node.getElementsByTagName("%s_item" % t.__name__)
            for item_node in item_nodes:
                k = str(item_node.getAttribute("key"))
                v = self.handleValue(item_node)
                value[k] = v
        elif MethodType is t:
            method_name = getText(value_node.childNodes)
            value = MethodField.Method(method_name)
        else:
            value = getText(value_node.childNodes)
            if IntType is t:
                value = int(value)
        return value

    def handleTales(self, tales_node, field):
        tal_nodes = tales_node.getElementsByTagName("tal")
        tales = {}
        for tal_node in tal_nodes:
            name = str(tal_node.getAttribute("name"))
            value = self.handleValue(tal_node)
            if value:
                tales[name] = TALESField.TALESMethod(str(value))
        field.tales = tales

    def handleOverrides(self, overrides_node, field):
        override_nodes = overrides_node.getElementsByTagName("override")
        overrides = {}
        for override_node in override_nodes:
            name = str(override_node.getAttribute("name"))
            value = self.handleValue(override_node)
            if value:
                overrides[name] = value
        field.overrides = overrides
        
    def handleMessages(self, messages_node, field):
        message_nodes = messages_node.getElementsByTagName("message")
        messages = {}
        for message_node in message_nodes:
            name = str(message_node.getAttribute("name"))
            value = self.handleValue(message_node)
            if value:
                messages[name] = value
        field.messages = messages
                
                    
class FormulatorExporter:
    def exportXML(self, form, stream=None):
        doc = minidom.Document()
        form_node = doc.createElement("form")
        form_node.setAttribute("id", form.getId())
        form_title = unicode(str(form.title), LATIN1)
        form_node.setAttribute("title", form_title.encode(UTF8))
        
        settings_form = form.settings_form
        settings = {}
        for field in settings_form.get_fields():
            key = field.getId()
            settings[key] = getattr(form, key, None)
        settings_node = doc.createElement("settings")
        for key, value in settings.items():
            n = doc.createElement("setting")
            n.setAttribute("name", key)
            self.exportValue(doc, n, value)
            settings_node.appendChild(n)
        form_node.appendChild(settings_node)
        groups = form.get_groups()
        groups_node = doc.createElement("groups")
        for group in groups:
            group_node = doc.createElement("group")
            group_node.setAttribute("name", group)
            fields = form.get_fields_in_group(group)
            fields_node = doc.createElement("fields")
            for field in fields:
                field_node = doc.createElement("field")
                field_name = unicode(str(field.getId()), LATIN1)
                field_node.setAttribute("name", field_name.encode(UTF8))
                meta_type = unicode(str(field.meta_type), LATIN1)
                field_node.setAttribute("meta_type", meta_type.encode(UTF8))
                
                values = field.values
                tales = getattr(field, 'tales', {})
                overrides = field.overrides
                messages = field.message_values
                values_node = doc.createElement("values")
                for key, value in values.items():
                    value_node = doc.createElement("value")
                    value_node.setAttribute("name", key)
                    self.exportValue(doc, value_node, value)
                    values_node.appendChild(value_node)
                field_node.appendChild(values_node)
                
                tales_node = doc.createElement("tales")
                for key, value in tales.items():
                    tal_node = doc.createElement("tal")
                    tal_node.setAttribute("name", key)
                    text = getattr(value, '_text', value)
                    text = unicode(str(text), LATIN1)
                    t = doc.createTextNode(text.encode(UTF8))
                    tal_node.appendChild(t)
                    tales_node.appendChild(tal_node)
                field_node.appendChild(tales_node)
                
                overrides_node = doc.createElement("overrides")
                for key, value in overrides.items():
                    override_node = doc.createElement("override")
                    override_node.setAttribute("name", key)
                    text = unicode(str(value), LATIN1)
                    t = doc.createTextNode(text.encode(UTF8))
                    override_node.appendChild(t)
                    overrides_node.appendChild(override_node)
                field_node.appendChild(overrides_node)

                messages_node = doc.createElement("messages")
                for key, value in messages.items():
                    message_node = doc.createElement("message")
                    message_node.setAttribute("name", key)
                    text = unicode(str(value), LATIN1)
                    t = doc.createTextNode(text.encode(UTF8))
                    message_node.appendChild(t)
                    messages_node.appendChild(message_node)
                field_node.appendChild(messages_node)
                fields_node.appendChild(field_node)
            group_node.appendChild(fields_node)
            groups_node.appendChild(group_node)
        form_node.appendChild(groups_node)
        doc.appendChild(form_node)
        if stream:
            stream.write(doc.toprettyxml())
            retval = None
        else:
            retval = doc.toprettyxml()
        doc.unlink()
        return retval
    
    def exportValue(self, doc, value_node, value):
        t = type(value)
        value_node.setAttribute("type", t.__name__)
        if ListType is t or TupleType is t:
            list_node = doc.createElement(t.__name__)
            for item in value:
                item_node = doc.createElement("%s_item" % t.__name__)
                self.exportValue(doc, item_node, item)
                list_node.appendChild(item_node)
            value_node.appendChild(list_node)
        elif DictType is t:
            dict_node = doc.createElement(t.__name__)
            for k, v in value.items():
                item_node = doc.createElement("%s_item" % t.__name__)
                item_node.setAttribute("key", k)
                self.exportValue(doc, item_node, v)
                dict_node.appendChild(item_node)
            value_node.appendChild(dict_node)
        elif MethodType is t:
            text = unicode(str(value.method_name), LATIN1)
            t = doc.createTextNode(text.encode(UTF8))
            value_node.appendChild(t)
        else:
            text = unicode(str(value), LATIN1)
            t = doc.createTextNode(text.encode(UTF8))
            value_node.appendChild(t)
        
        

                
                
if __name__ == '__main__':
    filename = "testform"
    importer = FormulatorImporter()
    
    form = importer.importXML(filename)
    
    
